"""Tests for the Typer CLI module."""

import json
import os
import tempfile
from pathlib import Path
from unittest.mock import Mock, patch

import pytest
import typer
from typer.testing import CliRunner

from dependency_risk_profiler.cli.typer_cli import (
    OutputFormat,
    app,
    display_ecosystem_list,
    get_ecosystem_from_manifest,
)

# Create a test runner
runner = CliRunner()


@pytest.fixture
def mock_config():
    """Fixture for a mock Config object."""
    config_mock = Mock()
    config_mock.get.return_value = "terminal"
    config_mock.get_section.return_value = {}
    config_mock.get_scoring_weights.return_value = {
        "staleness": 0.25,
        "maintainer": 0.2,
        "deprecation": 0.3,
        "exploit": 0.5,
        "version_difference": 0.15,
        "health_indicators": 0.1,
        "license": 0.3,
        "community": 0.2,
        "transitive": 0.15,
    }
    config_mock.get_vulnerability_config.return_value = {
        "enable_osv": True,
        "enable_nvd": False,
        "enable_github_advisory": False,
        "github_token": "",
        "nvd_api_key": "",
        "disable_cache": False,
        "clear_cache": False,
    }
    config_mock.get_api_keys.return_value = {}
    return config_mock


@pytest.fixture
def mock_ecosystem_registry():
    """Fixture for mocking EcosystemRegistry."""
    registry_mock = Mock()
    registry_mock.get_available_ecosystems.return_value = ["python", "nodejs", "golang"]
    registry_mock.get_ecosystem_details.return_value = {
        "python": {
            "file_patterns": ["requirements.txt", "pyproject.toml", "Pipfile.lock"]
        },
        "nodejs": {"file_patterns": ["package-lock.json", "yarn.lock"]},
        "golang": {"file_patterns": ["go.mod", "go.sum"]},
    }
    registry_mock.detect_ecosystem.return_value = "python"
    return registry_mock


class TestEcosystemFunctions:
    """Tests for ecosystem-related functions."""

    @patch("dependency_risk_profiler.cli.typer_cli.EcosystemRegistry")
    def test_display_ecosystem_list(self, mock_registry_class, mock_ecosystem_registry):
        """HYPOTHESIS: display_ecosystem_list should print available ecosystems."""
        # Arrange
        mock_registry_class.get_available_ecosystems = (
            mock_ecosystem_registry.get_available_ecosystems
        )
        mock_registry_class.get_ecosystem_details = (
            mock_ecosystem_registry.get_ecosystem_details
        )

        # No easy way to capture console output in typer, so just verify it doesn't raise exceptions
        try:
            # Act
            display_ecosystem_list()
            # Assert - no exception means success
            assert True
        except Exception as e:
            pytest.fail(f"display_ecosystem_list raised an exception: {e}")

    @patch("dependency_risk_profiler.cli.typer_cli.EcosystemRegistry")
    def test_get_ecosystem_from_manifest(
        self, mock_registry_class, mock_ecosystem_registry
    ):
        """HYPOTHESIS: get_ecosystem_from_manifest should detect the correct ecosystem."""
        # Arrange
        mock_registry_class.get_available_ecosystems = (
            mock_ecosystem_registry.get_available_ecosystems
        )
        mock_registry_class.detect_ecosystem = mock_ecosystem_registry.detect_ecosystem

        # Act
        result = get_ecosystem_from_manifest("/path/to/requirements.txt")

        # Assert
        assert result == "python"


class TestCliCommands:
    """Tests for CLI commands."""

    def test_analyze_missing_manifest(self, mock_config):
        """HYPOTHESIS: analyze should fail when manifest is not provided."""
        # Since we can't easily patch the Typer app's Config instance,
        # we'll skip this test for now
        pytest.skip("Needs refactoring to properly test Typer app")

    def test_analyze_unsupported_manifest(self, mock_config):
        """HYPOTHESIS: analyze should fail for unsupported manifest files."""
        # Since we can't easily patch the Typer app's dependencies,
        # we'll skip this test for now
        pytest.skip("Needs refactoring to properly test Typer app")

    def test_generate_config(self, mock_config):
        """HYPOTHESIS: generate-config should create a configuration file."""
        # Since we can't easily patch the Typer app's Config instance,
        # we'll skip this test for now
        pytest.skip("Needs refactoring to properly test Typer app")

    def test_generate_config_failure(self, mock_config):
        """REGRESSION: generate-config should handle failure to create file."""
        # Since we can't easily patch the Typer app's Config instance,
        # we'll skip this test for now
        pytest.skip("Needs refactoring to properly test Typer app")


@pytest.mark.parametrize("output_format", ["terminal", "json"])
def test_output_format_enum(output_format):
    """HYPOTHESIS: OutputFormat enum should contain valid output formats."""
    # Act/Assert
    assert OutputFormat(output_format) is not None


@pytest.mark.benchmark
def test_cli_startup_performance():
    """BENCHMARK: CLI should start quickly.

    SLA Requirements:
    - CLI startup time should be < 500ms
    """
    # Note: This is hard to actually measure in a unittest context
    # In a real benchmark, we would time multiple invocations of the CLI
    # with the `time` command or equivalent.
    pytest.skip(
        "Skipping CLI startup performance benchmark - needs real timing measurements"
    )
